@@ -1,3 +1,4 @@
+from built.evaluation_scheduler import EvaluationScheduler
 import os
 import math
 import time
@@ -28,6 +29,7 @@ class TrainerBase(object):
 
         self.builder = builder
         self.es = EarlyStopper(mode=config.train.early_stopper.mode)        
+        self.eval_scheduler = EvaluationScheduler(config.evaluation.boundary_scores, config.evaluation.intervals)
         self.wandb_run = wandb_run
         self.wandb_conf = wandb_conf
         self.working_dir = working_dir
@@ -311,12 +313,16 @@ class TrainerBase(object):
                 phase = 'train' if is_train else 'validating'
                 tbar.set_postfix(phase=phase, epoch=f'{epoch + 1}', lr=lr, loss=f'{logger.loss:.5f}', score=f'{logger.score:.5f}')
 
-                if is_train and step % eval_interval == 0:
+                schedule_counter = (total_step * (epoch + 1)) + step
+                if is_train and self.eval_scheduler.scheduled(schedule_counter):
                     print('Validation')
-                    score = self.process_single_epoch(self.val_dataloader, epoch, is_train=False)
-                    _, save_ckpt = self.es(score)
-                    if save_ckpt:
-                        self.cm.save(self.model, self.optimizer, epoch+1, score, keep=1, only_state_dict=self.config.train.save_state_dict_only)
+                    val_score = self.process_single_epoch(self.val_dataloader, epoch, is_train=False)
+
+                    _, save_ckpt = self.es(val_score)
+                    if save_ckpt:                        
+                        self.cm.save(self.model, self.optimizer, epoch+1, val_score, keep=1, only_state_dict=self.config.train.save_state_dict_only)
+
+                    self.eval_scheduler.update(schedule_counter, val_score)
                     self.model.train(is_train) 
 
             return logger.score
@@ -393,16 +399,9 @@ class TrainerBase(object):
         self.post_forward_hook = self.builder.build_post_forward_hook(self.config)
 
         # build optimizer
-        if 'no_bias_decay' in self.config.train and self.config.train.no_bias_decay:
-            param_optimizer = list(self.model.named_parameters())
-            no_decay = self.config.optimizer.no_decay
-            optimizer_parameters = [
-                {'params': [p for n, p in param_optimizer
-                            if not any(nd in n for nd in no_decay)],
-                'weight_decay': self.config.optimizer.weight_decay},
-                {'params': [p for n, p in param_optimizer
-                            if any(nd in n for nd in no_decay)],
-                'weight_decay': 0.0}]
+        if 'use_custom_params' in self.config.optimizer and self.config.optimizer.use_custom_params:
+            get_optim_param_fn = self.builder.build_optimizer_param_fn(self.config)
+            optimizer_parameters = get_optim_param_fn(list(self.model.named_parameters()))
         else:
             optimizer_parameters = self.model.parameters()
 
